<?php

// Note the comments in this program are made to be read by phpDocumentor
// see http://useraccess.craftysyntax.com/

/**    
 * =====================================================================
 * DESCRIPTION:
 *
 *   User Access is a multiple application authentication permission 
 *   framework designed to allow users to log into multiple different 
 *   applications across multiple servers with one log in. 
 *   Like LiveUser the system runs off of associative array containers 
 *   allowing the application to be flexible and easily extendable. 
 *   User Access can also be used for a single application authentication.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For questions, help, comments, discussion, etc., 
 * Please Visit http://www.craftysyntax.com
 *
 * You may contact the author of User Access by e-mail at:
 * eric@craftysyntax.com
 *
 * Or, write to:
 * Eric Gerdes
 * P.O. Box 1457 
 * Puunene Hi 96784
 * 
 * The latest version of Smarty can be obtained from:
 * http://craftysyntax.com/
 *
 * @package  UserAccess
 * @category authentication
 * @link http://craftysyntax.com/
 * @copyright 2004 Eric Gerdes
 * @author Eric Gerdes <eric@craftysyntax.com>
 * @version 0.4.1 
 */

/** 
  * Description: output a readable error message
  * @param $object: the DB object    
  */
function handle_pear_error($object)
	{
	  print "[ " . $object->getMessage() . "] ( " . $object->getDebugInfo() . " <BR>\n";
	  exit;
	}


Class User_Access
{
    /**
     * user id OR username (whatever the primary key for the user is)     
     * @var string
     */
      var $USER_ID = "";
    /**
     * username     
     * @var string
     */      
      var $USERNAME = "";
    /**
     * password     
     * @var string
     */           
      var $PASSWORD = "";
    /**   
     * the user identification set as teh session id.
     * @var string
     */           
      var $SESSIONID = "";
    /**   
     * if the user is logged in or not.
     * @var boolean
     */     
      var $LOGGEDIN = false;
    /**   
     * remote ip address of the user.
     * @var string
     */           
      var $IP_ADDR  = "";
    /**   
     * browser User Agent.
     * @var string
     */         
      var $USER_AGENT   = "";
      
    /**   
     * Allow loggins based on ipaddress and user agent if sessions and 
     * cookies fail.. by itself this is not a good idea but useful in some cases.
     * @var string
     */         
      var $ALLOW_IP_BASED_LOGGIN  = true;      
    /**   
     * last referer.
     * @var string
     */       
      var $REFERER    = "";
    /**   
     * pointer to the User database 
     * @var object
     */         
      var $USERDB = "";
    /**   
     * poniter to the session database
     * @var object
     */         
      var $SESSIONDB ="";
    /**   
     * pointer to the permission database
     * @var object
     */      
      var $PERMISSIONDB ="";
    /**   
     * current error message.
     * @var string
     */          
      var $ERROR = ""; 
    /**   
     *  DB row of the user 
     * @var array
     */          
      var $USER_ARRAY = array();
    /**   
     *  DB row of the session
     * @var array
     */        
      var $SESS_ARRAY = array();
    /**   
     *  http page of the log in page
     * @var string
     */  
      var $LOGINPAGE = "";
    /**   
     *  http page of the log out page
     * @var string
     */       
      var $LOGOUT = "";
    /**   
     * The assicative array that contains the authintication information
     * defined in ua.config.php 
     * AUTH_INFO
     * |
     * -- type     - The type of authenticate for the user. This could be:
     * |            "FTP" - use FTP to authenticate.
     * |            "USER_INFO" - use the USER_INFO assicative array for authentication.
     * -- ftp_host - The URL of the host to connect to if using FTP (ie. www.myhost.com ).           
     *
     * @var array
     */       
      var $AUTH_INFO = array();
    /**   
     * USER_INFO - The assicative array that contains the user information : 
     * |
     * -- type   -  The type of user authentication: this can only be DB at this time.
     * |
     * -- myencription - The type of encription for the password field. This could be:
     * |                  ""    - no encription
     * |                  "MD5" - md5 encription
     * |                  "ENCODE" - ecode/decode
     * -- encodekey - the salt encode key for the ENCODE encription.
     * |
     * -- dsn - the dsn connection information
     * |
     * -- dbtable - the name of the database table that holds the user information
     * |
     * -- dbfields - the map of what fields are called in that table.
     *       |
     *       --user_id  - the name of the user_id field. this Should be the primary key 
     *       |            for the user. It is crosslisted to the rest of the tables.
     *       --username - the username field. 
     * @var array     
     */  
      var $USER_INFO = array();
   /** 
     * @var array     
     */        
      var $SESSION_INFO = array();           
   /** 
     * @var array     
     */          
      var $PERMISSION_INFO = array();                           

/** 
  * Description: constructor for the class..
  * @param $UA_Config: the associative array containing the configuration for
  *                   this object. See ua.config.php  
  */
function User_Access($UA_Config){

   $this->LOGINPAGE = $UA_Config['LOGINPAGE'];
   $this->LOGOUT = $UA_Config['LOGOUT'];
     
   $this->AUTH_INFO = $UA_Config['AUTH_INFO'];   
   $this->USER_INFO = $UA_Config['USER_INFO'];  
   $this->SESSION_INFO = $UA_Config['SESSION_INFO'];
   $this->PERMISSION_INFO = $UA_Config['PERMISSION_INFO'];    

   // connect to the databases
   $this->dbconnect();    

   // Identify the user based on session and env.
   $this->identity();
}

/** 
  * Description: Gets the Identity of this user and sets a session for them. 
  */
function identity(){
   global $HTTP_POST_VARS,$HTTP_GET_VARS,$_SESSION,$_COOKIE,$REMOTE_ADDR,$HTTP_SERVER_VARS,$HTTP_ENV_VARS,$HTTP_USER_AGENT,$HTTP_REFERER;
    
    $session_info = $this->SESSION_INFO; 
    $session_dbfields = $session_info['dbfields'];
    $sessionfields = $session_info['dbfields'];
     
   // set the session information.
   if(isset($HTTP_POST_VARS['PHPSESSID']))
      session_id($HTTP_POST_VARS['PHPSESSID']);
      
   if(isset($HTTP_GET_VARS['PHPSESSID']))
      session_id($HTTP_GET_VARS['PHPSESSID']);
   
   // alternate session setting 
   if(isset($HTTP_POST_VARS['utoken']))
      session_id($HTTP_POST_VARS['utoken']);
      
   if(isset($HTTP_GET_VARS['utoken']))
      session_id($HTTP_GET_VARS['utoken']);
   
   // cookie
   if(isset($_COOKIE['utoken'])){
   	session_id($_COOKIE['utoken']);
   }   	
               
   // get their ip address Code from PHPBB2
  if( getenv('HTTP_X_FORWARDED_FOR') != '' ){
    $client_ip = ( !empty($HTTP_SERVER_VARS['REMOTE_ADDR']) ) ? $HTTP_SERVER_VARS['REMOTE_ADDR'] : ( ( !empty($HTTP_ENV_VARS['REMOTE_ADDR']) ) ? $HTTP_ENV_VARS['REMOTE_ADDR'] : $REMOTE_ADDR );
    $entries = explode(',', getenv('HTTP_X_FORWARDED_FOR'));
    reset($entries);
    while (list(, $entry) = each($entries)){
	$entry = trim($entry);
	if ( preg_match("/^([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)/", $entry, $ip_list) ){
		$private_ip = array('/^0\./', '/^127\.0\.0\.1/', '/^192\.168\..*/', '/^172\.((1[6-9])|(2[0-9])|(3[0-1]))\..*/', '/^10\..*/', '/^224\..*/', '/^240\..*/');
		$found_ip = preg_replace($private_ip, $client_ip, $ip_list[1]);
		if ($client_ip != $found_ip){
			$client_ip = $found_ip;
			break;
		}
	}
     }
   } else {
      $client_ip = ( !empty($HTTP_SERVER_VARS['REMOTE_ADDR']) ) ? $HTTP_SERVER_VARS['REMOTE_ADDR'] : ( ( !empty($HTTP_ENV_VARS['REMOTE_ADDR']) ) ? $HTTP_ENV_VARS['REMOTE_ADDR'] : $REMOTE_ADDR );
   }
 
   $client_agent = ( !empty($HTTP_SERVER_VARS['HTTP_USER_AGENT']) ) ? $HTTP_SERVER_VARS['HTTP_USER_AGENT'] : ( ( !empty($HTTP_ENV_VARS['HTTP_USER_AGENT']) ) ? $HTTP_ENV_VARS['HTTP_USER_AGENT'] : $HTTP_USER_AGENT );
   $client_referer = $HTTP_REFERER;

    // ALLOW_IP_BASED_LOGGIN
    if($this->ALLOW_IP_BASED_LOGGIN){
     $query = "SELECT * FROM " . $session_info['dbtable'] . " WHERE " . $sessionfields['ipaddress'] . "='$client_ip' AND " . $sessionfields['useragent'] . "='". $client_agent . "'";
     $sth_session = $this->SESSIONDB->query($query);
     if (DB::isError($this->SESSIONDB)) {
         die ($this->SESSIONDB->getMessage());
     } 
     if($sth_session->numrows() != 0){     
       $row = $sth_session->fetchRow(DB_FETCHMODE_ASSOC);      
       $sessionid = $sessionfields['sessionid'];
       $existingsession =  $row[$sessionid];
       session_id($existingsession);
     }
    }     
    
    // start the session..
    session_start();


 if(!(isset($utoken))){ $utoken = session_id(); }

 $this->IP_ADDR = $client_ip;
 $this->USER_AGENT = $client_agent;
 $this->REFERER = $client_referer;   
 $this->SESSIONID = session_id();
}


/**
  *Description: disconnect from the database for the users table and the sessions 
  *             table and permissions table.
  */
function dbdisconnect(){
	if($this->USERDB){ $this->USERDB->disconnect(); }
	if($this->SESSIONDB){ $this->SESSIONDB->disconnect(); }
	if($this->PERMISSIONDB){ $this->PERMISSIONDB->disconnect(); }
}

/**
  * Description: connect to the database for the users table and the sessions 
  *              table and permissions table.
  */
function dbconnect(){  
  
  $user_info = $this->USER_INFO;
  $userdsn = $user_info['dsn'];
  $sess_info = $this->SESSION_INFO;  
  $sessdsn = $sess_info['dsn'];
  $permission_info = $this->PERMISSION_INFO;  
  $permissiondsn = $permission_info['dsn'];       

  //connection for user DB
  if($user_info['type'] == "DB"){	
     // Data Source Name: This is the universal connection string
     $dsn_user = $userdsn['dbtype'] . "://" . $userdsn['username'] . ":" . $userdsn['password']. "@" . $userdsn['host'] . "/" . $userdsn['database'];

     $USERDB = DB::connect("$dsn_user");
     $this->USERDB = $USERDB;

     if (DB::isError($this->USERDB)) {
        die ($this->USERDB->getMessage());
      }    
     $this->USERDB->setErrorHandling(PEAR_ERROR_CALLBACK, 'handle_pear_error');     
   }  
  
  //connection for session DB
  if($sess_info['type'] == "DB"){	
     // Data Source Name: This is the universal connection string
     $dsn_session = $sessdsn['dbtype'] . "://" . $sessdsn['username'] . ":" . $sessdsn['password']. "@" . $sessdsn['host'] . "/" . $sessdsn['database'];
     $SESSIONDB = DB::connect($dsn_session);
     $this->SESSIONDB = $SESSIONDB;
     if (DB::isError($this->SESSIONDB)) {
        die ($this->SESSIONDB->getMessage());
      }    
     $this->SESSIONDB->setErrorHandling(PEAR_ERROR_CALLBACK, 'handle_pear_error'); 
   }   
   
  //connection for permission DB 
  if($permission_info['type'] == "DB"){	
     // Data Source Name: This is the universal connection string
     $dsn_permission = $permissiondsn['dbtype'] . "://" . $permissiondsn['username'] . ":" . $permissiondsn['password']. "@" . $permissiondsn['host'] . "/" . $permissiondsn['database'];
     $this->PERMISSIONDB = DB::connect($dsn_permission);
     if (DB::isError($this->PERMISSIONDB)) {
        die ($this->PERMISSIONDB->getMessage());
      }
     $this->PERMISSIONDB->setErrorHandling(PEAR_ERROR_CALLBACK, 'handle_pear_error');      
   }   
		
		
}

/**
  * Description: sets the username and password for the user in this object based on 
  *               utoken, or username and password.
  * @param  username  : The username of the authinicated user can be null if utoken is set
  * @param  password  : The password of the authinicated user can be null if utoken is set
  */
function init($username=null,$password=null){
    global $PHP_AUTH_USER,$PHP_AUTH_PW,$_SESSION; 

    $sessionid = $this->SESSIONID;      
    $loginpage = $this->LOGINPAGE;
    $session_info = $this->SESSION_INFO; 
    $session_dbfields = $session_info['dbfields'];
    $user_id_field_session = $session_dbfields['user_id'];
    $ENCODEKEY = $session_info['encodekey'];
    $user_info = $this->USER_INFO; 
    $user_dbfields = $user_info['dbfields'];
    $usertable = $user_info['dbtable'];
    $user_field = $user_dbfields['username'];
    $user_id_field_user= $user_dbfields['user_id'];
    
    // if the username and the password was passed to us set them then validate.
    if( ($username!="") && ($password!="") ){ 
    	$this->USERNAME = $username;
        $this->PASSWORD = $password;
        $this->validate(); 
        $this->update_session();             
    }  

    // if the database session does not exists create it.. 
    $this->create_session();  
   
    // if we have a vaild session and do not have a username and password and 
    // we are not using PHP_AUTH get the username, password and user_id from session database.    
    $validsession = $this->validsession();                 
    if( ($loginpage != "") && (($username=="") || ($password=="")) && $validsession==1){        

       $query = "SELECT * FROM " . $session_info['dbtable'] . " WHERE " . $session_dbfields['sessionid']. "='$sessionid'";
       $rs = $this->SESSIONDB->query($query);
       if($rs->numRows() != 0){        
         
         $row = $rs->fetchRow(DB_FETCHMODE_ASSOC);
         $user_id = $row[$user_id_field_session];

         //get the username and user DB.
         $query = "SELECT * FROM $usertable WHERE $user_id_field_user='$user_id'";
         $res = $this->USERDB->query($query);
         
         $USER_ARRAY = $res->fetchRow(DB_FETCHMODE_ASSOC);
         $username = $USER_ARRAY[$user_field];
         $this->USER_ARRAY = $USER_ARRAY;
 
         //decode password if used... 
         if($session_dbfields['password'] != ""){
           if(isset($row[$session_dbfields['password']])){
               $encodedpass = $row[$session_dbfields['password']]; 
           } else { $encodedpass = ""; }
           $q = "SELECT DECODE(\"" . addslashes($encodedpass) . "\",\"" . $ENCODEKEY . "\") as password";
           $rs = $this->SESSIONDB->query($q);
           $row = $rs->fetchRow(DB_FETCHMODE_ASSOC);
           $password = $row['password'];
         }
         
         // if we got this far we are logged in.
         $this->LOGGEDIN = true;
       } else {
       	 $this->LOGGEDIN = false;
       	 $this->ERROR = "invalid session";
       }
     }  
    // gather username and password from PHP_AUTH if we are using that.
    if($loginpage == ""){
         $username = $PHP_AUTH_USER;
         $password = $PHP_AUTH_PW;
         $user_id = $username;         
    }    

    // set values for the user.
        
    if(isset($user_id)){ $this->USER_ID = $user_id; }
    if($this->USER_ID != ""){ $user_id = $this->USER_ID; }
    $this->USERNAME = $username;
    $this->PASSWORD = $password;
    
    // set 
    if(!(isset($username))){ $username = ""; }
    if(!(isset($user_id))){ $user_id = ""; }    
    
    // if no username then we are not logged in..
    if( ( ($username == "0") || ($user_id == "0")) || ( ($username == "") && ($user_id == ""))){                  
       $this->LOGGEDIN = false;
       $this->ERROR = "NO USERNAME OR USER ID for " . $this->SESSIONID;
    }                
  $this->update_session();           
}

/** 
  * Description: returns true if the session id passed to it is still valid.. 
  * @param  username  : The username of the authinicated user. 
  * @return  bool  true if valid false if not valid.
  */
function validsession(){

 $sessionid = $this->SESSIONID;
 $sessiondb = $this->SESSIONDB;
 $session_info = $this->SESSION_INFO;
 $sessionfields = $session_info['dbfields'];
   
 $rightnow = date("YmdHis");
        
 $query = "SELECT * FROM " . $session_info['dbtable'] . " WHERE " . $sessionfields['sessionid'] . "='$sessionid' AND " . $sessionfields['accessexpire'] . ">$rightnow ";
 $sth = $sessiondb->query($query);
 if( $sth->numrows() == 0){ 
     $this->LOGGEDIN = false;
     return false;
  } else {	
     $row = $sth->fetchRow(DB_FETCHMODE_ASSOC);
     $this->SESS_ARRAY = $row;
     return true;
  }
    
}

/**
  * Description: Deletes Expired user sessions from database.
  */
function expireoldsessions(){

 $sessionid = $this->SESSIONID;
 $sessiondb = $this->SESSIONDB;
 $session_info = $this->SESSION_INFO;
 $sessionfields = $session_info['dbfields'];
   
 $rightnow = date("YmdHis");
        
 $query = "DELETE FROM " . $session_info['dbtable'] . " WHERE " . $sessionfields['accessexpire'] . "<$rightnow ";
 $sessiondb->query($query);
      
}

/**
  * Description: validates the username and password against user connection.
  * @return  bool  true if valid false if not valid.
  */
function validate(){
 
  $username = $this->USERNAME;
  $password =  $this->PASSWORD;
  $sessionid = $this->SESSIONID;
  $auth_info = $this->AUTH_INFO;  
  $user_info = $this->USER_INFO;
  $sess_info = $this->SESSION_INFO;  
  $userfields = $user_info['dbfields'];
  $encoding = $user_info['myencription'];
  $user_id_field_user = $userfields['user_id'];
  
  if( ($username == "") || ($password == "")) { return false; }
   
  //place holder for socket authentication... 
  if($auth_info['type'] == "SOCKET"){
     // some day.. 
  }

  // default the result of the validation to false (-1)
  $result = -1; 
  
  // if using a FTP connection to Validate the user 
  if($auth_info['type'] == "FTP"){   
    error_reporting(0);
    $result = 1;
    // set up basic connection
    $conn_id = ftp_connect($auth_info['ftp_host']);
    // login with username and password
    $login_result = ftp_login($conn_id, $username, $password);
    // check connection
    if ((!$conn_id) || (!$login_result)) {
       $result = -1;
    }
    if($password ==""){ $result = -1; }
    ftp_close($conn_id);
    error_reporting(E_ALL & ~E_NOTICE);      
    $user_id = $username;
  }
  
  if($auth_info['type'] == "USER_INFO")  {
   // if using a DB or txt-db-api connection to Validate the user 
   if( ($user_info['type'] == "DB") || ($user_info['type'] == "txt-db-api") ){   
     // check username and password against database table.
     switch($encoding){
       case "MD5" :
          $hash = md5($password);
          $query = "SELECT * FROM " . $user_info['dbtable'] . " WHERE " . $userfields['username'] . "='$username' AND " . $userfields['password'] . "='$hash'";
          break;
       case "PASSWORD" :
          $query = "SELECT * FROM " . $user_info['dbtable'] . " WHERE " . $userfields['username'] . "='$username' AND " . $userfields['password'] . "=PASSWORD('$password')";          
          break;
       default :
          $query = "SELECT * FROM " . $user_info['dbtable'] . " WHERE " . $userfields['username'] . "='$username' AND " . $userfields['password'] . "='$password'";
          break;       
      } 
     $sth = $this->USERDB->query($query);
     if ($sth->numrows() < 1){          
       $result = -1;
       $this->ERROR = " INVALID USER/PASSWORD ";
     } else {
       $result = 1;   
       $row = $sth->fetchRow(DB_FETCHMODE_ASSOC);
       $this->USER_ARRAY = $row;
       if($row[$user_id_field_user] != ""){
         $this->USER_ID = $row[$user_id_field_user];
       }        
     }
    } // end using DB or txt-db-api
   } // end if using USER_INFO
  
  // if everything is ok 
  if($result == 1){
     $this->LOGGEDIN = true;    
     if(isset($user_id)){
        $this->USER_ID = $user_id;        
     }    
     return true;
  } else {
     $this->LOGGEDIN = false; 
     return false;
  }  

}	

/**
  * Description: Updates the timeout for the session 
  * @param  noexpire  : if set to 1 then the session will be set to not expire. 
  */
function update_session($noexpire=""){  
 global $_COOKIE;
 
 $sessionid = $this->SESSIONID;   
 $user_id = $this->USER_ID;
 $password = $this->PASSWORD;
 $sessiondb = $this->SESSIONDB;
 $session_info = $this->SESSION_INFO;
 $keeppass = $session_info['storepass']; 
 $ENCODEKEY = $session_info['encodekey']; 
 $sessionfields = $session_info['dbfields'];
 
    if($keeppass ==1){
    	$passwordupdate = "," . $sessionfields['password'] . "=ENCODE(\"$password\",\"$ENCODEKEY\")";
    } else { 
    	$passwordupdate = ""; 
    }

  if($noexpire == ""){
    $twentymin  = date("YmdHis",mktime(date("H"),date("i")+40,date("s"),date("m"),date("d"),date("Y")));  
  } else {
    $twentymin  = 20781221192374;
  }
  
  $environment = "";
  if($sessionfields['ipaddress'] != ""){ $environment = $sessionfields['ipaddress'] . "='" . $this->IP_ADDR . "',"; }
  if($sessionfields['useragent'] != ""){ $environment .= $sessionfields['useragent'] . "='" . addslashes($this->USER_AGENT) . "',"; }
  if($sessionfields['lastaccess'] != ""){ $environment .= $sessionfields['lastaccess'] . "='" . date("YmdHis") . "',"; }

  if($sessionfields['referer'] != ""){ 
  	$referer_a = split("/",$sessionfields['referer']);
  	if( isset($referer_a[2]) ){
   	 if($_SERVER['HTTP_HOST'] != $referer_a[2] ){
  	   $environment .= $sessionfields['referer'] . "='" . addslashes($this->REFERER) . "',"; 
  	 }
  	}
  }   
 
  $query = "SELECT * FROM " . $session_info['dbtable'] . " WHERE " . $sessionfields['sessionid'] . " ='$sessionid'"; 
  $rs = $this->SESSIONDB->query($query);
  $row = $rs->fetchRow(DB_FETCHMODE_ASSOC);
  $expirefield = $sessionfields['accessexpire'];
  $currentexpire = $row[$expirefield];
    
  if($currentexpire < $twentymin){
    $accessupdate =  "," . $sessionfields['accessexpire'] . "='$twentymin'";
  } else { $accessupdate =  ""; }
  $query = "UPDATE " . $session_info['dbtable'] . " set " . $environment . $sessionfields['user_id'] . "='" . $user_id . "' $accessupdate $passwordupdate WHERE " . $sessionfields['sessionid'] . " ='$sessionid'";
  $this->SESSIONDB->query($query);

}

/**
  * Description: Creates the session and the time till the session expires.
  */
function create_session(){  
  
  $user_id = $this->USER_ID;
  $username = $this->USERNAME;
  $password =  $this->PASSWORD;
  $sessionid = $this->SESSIONID;
  $session_info = $this->SESSION_INFO;
  $sessionfields = $session_info['dbfields'];
  $keeppass = $session_info['storepass'];   
  $ENCODEKEY = $session_info['encodekey'];
    
  $twentymin  = date("YmdHis",mktime(date("H"),date("i")+20,date("s"),date("m"),date("d"),date("Y")));
  $rightnow = date("YmdHis");         
  
  // see if a session is already open for this user..
  $query = "SELECT * FROM " . $session_info['dbtable'] . " WHERE " . $sessionfields['sessionid'] . "='$sessionid'";
  $sth_session = $this->SESSIONDB->query($query);
  if (DB::isError($this->SESSIONDB)) {
        die ($this->SESSIONDB->getMessage());
  }   
  
  // The session given does not exist.. . 
  if($sth_session->numrows() == 0){    

    // if we are storing the password.
    if($sessionfields['password'] == "YES"){ 
    	$mypasswordfield = ",`" . $sessionfields['password'] . "`"; 
    	$passwordvalue = ",ENCODE(\"$password\",\"$ENCODEKEY\")";
    } else { 
    	$mypasswordfield = ""; 
    	$passwordvalue = "";
    }
    
    $query = "INSERT INTO " . $session_info['dbtable'] . "(" . $sessionfields['user_id'] . "," . $sessionfields['sessionid'] . "," . $sessionfields['accessexpire'] . $mypasswordfield .") VALUES ('$user_id','$sessionid','$twentymin' $passwordvalue )";
    $this->SESSIONDB->query($query);
 
  }          

}

/**
  * Description: Logs the user out and directs to logout page if any.
  */
function logout(){
  
  $user_id = $this->USER_ID;
  $session_info = $this->SESSION_INFO;
  $sessionfields = $session_info['dbfields'];
  $logout = $this->LOGOUT;
    
  // delete all sessions for this user..
  $query = "DELETE FROM " . $session_info['dbtable'] . " WHERE " . $sessionfields['user_id'] . "='$user_id'";
  $this->SESSIONDB->query($query);    
  
  //destroy session variables and session
  session_unset();
  session_destroy();
  $this->LOGGEDIN = false;
  if($logout != ""){
     Header("Location: $logout");
  } else {
    print "log out successful";	
  }
  exit;
}

/**
  * Description: Authenticates the user 
  * @param  message  : message to relay to log in page (error message)
  */
function authenticate($message=""){
    
  $loginpage = $this->LOGINPAGE;
  $sessionid = $this->SESSIONID;
  $userid = $this->USER_ID;  
  $sessiondb = $this->SESSIONDB;
  $session_info = $this->SESSION_INFO;
  $sessionfields = $session_info['dbfields'];
   
  $query = "DELETE FROM " . $session_info['dbtable'] . " WHERE " . $sessionfields['user_id'] . "='$userid'";
  $this->SESSIONDB->query($query);  
    
  if($loginpage == ""){
    Header("WWW-Authenticate: Basic realm=\"Database Access\"");
    Header("HTTP/1.0 401 Unauthorized");
    readfile("noaccess.htm");
    exit;
  } else {
    $url = $loginpage . "?message=" . urlencode($message);
    Header("Location: $url");
    exit;
  }
}

/** 
  * Description: updates a field in either the Auth, session, or permission db
  * @param  container  : The type of container either : USER or SESSION or PERMISSION
  * @param  field  : The field name to update.
  * @param  value  : The value to update it to.
  * @param  user_id  : The user_id to update
  */
function updateval($container,$field,$value,$user_id=""){      
  $sessionid = $this->SESSIONID;
  $user_info = $this->USER_INFO;
  $usertable = $user_info['dbtable'];
  $userfields = $user_info['dbfields'];    
  $user_id_field = $userfields['user_id'];
  $sess_info = $this->SESSION_INFO;  
  $sesstable = $sess_info['dbtable'];
  $sessfields = $sess_info['dbfields'];   
  $utoken_session = $sessfields['sessionid'];
  $permission_info = $this->PERMISSION_INFO;  
  $permissiontable = $permission_info['user_rights_table'];  
  $permissionfields = $permission_info['user_rights_fields'];
   
  if($user_id == ""){ $user_id = $this->USER_ID; }
  
  switch($container){
   case "USER":
      $query = "UPDATE $usertable SET $field='$value' WHERE $user_id_field='$user_id'";
      $this->USERDB->query($query);
      break;
   case "SESSION":
      $query = "UPDATE $sesstable SET $field='$value' WHERE $utoken_session='$sessionid'";
      $this->SESSIONDB->query($query);         
      break;
   default:
      $query = "UPDATE $permissiontable SET $field='$value' WHERE $user_id_field='$user_id'";
      $this->PERMISSIONDB->query($query);   
      break;
  }

}

/**
  * Description: see if the user $user_id has access to $area with $permission rights.
  * @param  permission  : The username of the authinicated user. 
  * @param  area  : The area name to access
  * @param  user_id  : The user_id of who needs access if "" then use current id.
  * @return  bool  true if valid false if not valid.
  */
function has_access($permission,$area,$user_id=""){    
    $permission_info = $this->PERMISSION_INFO; 
    $user_rights_fields = $permission_info['user_rights_fields'];	

   if($user_id == ""){ $user_id = $this->USER_ID; }    
   $query = "SELECT * FROM " . $permission_info['user_rights_table'] . " WHERE " . $user_rights_fields['user_id'] . "='$user_id' AND " . $user_rights_fields['area_id'] . "='$area' AND " . $user_rights_fields['permission'] . "='$permission' ";
   $result = $this->PERMISSIONDB->query($query);
   if($result->numrows() > 0){
     return true;
   } else {
     return false;   	
   }
}
        
/** 
  * Description: sets utoken as a cookie to remember the user.
  */
function rememberuser(){
  $UTOKEN = $this->SESSIONID;
  setcookie ("utoken", $UTOKEN, time() + 993600);
  $this->update_session(1);
}

/** 
  * Description: returns the requested object value .. same as 
  *              $this->KEY; 
  * @param  key  : The variable name.
  * @return  mixed the object that is requested
  */
function getval($key){
  return $this->$key;
 }

/** 
  * Description: updates a field in the object.
  * @param  key  :  The variable name.
  * @param  val  :   The value 
  */
function setval($key,$val){
   $this->$key = $val;		
} 
 
} // end class.
?>
